/**
 ****************************************************************************
 * @file	initial.c
 * @brief	Initialize Functions
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include "ipdefine.h"
#include "mcuip_drv.h"
#include "D_Para.h"

#include "tmpm375_wdt.h"
#include "tmpm375_tmrb.h"
#include "tmpm375_gpio.h"
#include "tmpm375_adc.h"
#include "tmpm375_enc.h"

#define DEFINE_APP
#include "initial.h"
#undef  DEFINE_APP


/*===================================================================*
	  Constant Data Definition
 *===================================================================*/
/*  */
#define	INT4KH_TB_CH			TSB_TB0
#define	INT4KH_IRQ_INTTX		INTTB00_IRQn
#define INT4KH_IRQ_LEVEL		INT4KH_LEVEL

#define	TMRB_PRD_4KH			TMRB_DIV8_PRD(250)

/**
 *********************************************************************************************
 * @brief		Initialize 4kH interval Timer.
 *
 * @return		none
 *********************************************************************************************
 */
void init_Timer_interval4kH(void)
{
	TMRB_InitTypeDef m_tmrb;

	m_tmrb.Mode = TMRB_INTERVAL_TIMER;
	m_tmrb.ClkDiv = TMRB_CLK_DIV_8;
	m_tmrb.TrailingTiming = TMRB_PRD_4KH;						/* periodic time */
	m_tmrb.UpCntCtrl = TMRB_AUTO_CLEAR;
	m_tmrb.LeadingTiming = TMRB_PRD_4KH;						/* periodic time */

	TMRB_Enable(INT4KH_TB_CH);
	TMRB_Init(INT4KH_TB_CH, &m_tmrb);

	NVIC_SetPriority(INT4KH_IRQ_INTTX, INT4KH_IRQ_LEVEL);
	NVIC_ClearPendingIRQ(INT4KH_IRQ_INTTX);
	NVIC_EnableIRQ(INT4KH_IRQ_INTTX);

	TMRB_SetRunState(INT4KH_TB_CH, TMRB_RUN);
}


/**
 *********************************************************************************************
 * @brief		Initialize encoder
 *
 * @return		none
 *********************************************************************************************
 */
void init_ENCen(void)
{
#if defined(__USE_ENCODER)

    ENC_InitTypeDef m_enc;

	/***** For Encoder Setting ****************************************/

	/* ENC initialization */
	m_enc.ModeType = ENC_ENCODER_MODE;
	m_enc.PhaseType = ENC_TWO_PHASE;
	m_enc.CompareStatus = ENC_COMPARE_DISABLE;
	m_enc.ZphaseStatus = ENC_ZPHASE_ENABLE;
	m_enc.FilterValue = ENC_FILTER_VALUE127;
	m_enc.IntEn = ENC_INTERRUPT_DISABLE;
	m_enc.PulseDivFactor = ENC_PULSE_DIV1;

	ENC_Init(TSB_EN0, &m_enc);
	ENC_SetCounterReload(TSB_EN0, cENC_PULSE_NUM_CH1 * cENCMULTIPLE - 1);

	/**** port setting ****/
	/* GPIO initialization */
	GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_0, ENABLE);		/* Set PE0 as input */
	GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_3, GPIO_BIT_0);	/* Set PE0 as ENCA0 */
	GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_1, ENABLE);		/* Set PE1 as input */
	GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_3, GPIO_BIT_1);	/* Set PE1 as ENCB0 */
	GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_2, ENABLE);		/* Set PE2 as input */
	GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_3, GPIO_BIT_2);	/* Set PE2 as ENCZ0 */

	/* Enable ENC */
	ENC_Enable(TSB_EN0);

#endif /* __USE_ENCODER */
}


/**
 *********************************************************************************************
 * @brief		Initialize ADC
 *
 * @return		none
 *********************************************************************************************
 */
void init_ADCen(void)
{
	AD_InitTypeDef adini;

	adini.shunt = cSHUNT_TYPE_CH1;
	adini.iuch = cADCH_CURRENT_U_CH1;
	adini.ivch = cADCH_CURRENT_V_CH1;
	adini.iwch = cADCH_CURRENT_W_CH1;
	adini.idcch = cADCH_CURRENT_IDC_CH1;
	adini.vdcch = cADCH_VDC_CH1;
	adini.pmd_ch = cPMD1;
	adini.pints = cPINTS_B;
	IP_ADC_init(TSB_ADB, &adini);

#if defined(__MOTOR_DEBUG_INT)
	NVIC_SetPriority(INTADBPDB_IRQn, 2);
	NVIC_ClearPendingIRQ(INTADBPDB_IRQn);
	NVIC_EnableIRQ(INTADBPDB_IRQn);
#endif /* defined(__MOTOR_DEBUG) */


	/* Set ADC Unit A Clock */
	ADC_SetClk(TSB_ADB, ADC_HOLD_FIX, ADC_FC_DIVIDE_LEVEL_NONE);
	/* Enable ADC Unit B */
	ADC_Enable(TSB_ADB);

#if defined(__USE_INAMP)
	IP_AMP_init();
#endif /* __USE_INAMP */

}

/**
 *********************************************************************************************
 * @brief		Initialize PMD
 *
 * @return		none
 *********************************************************************************************
 */
void init_PMDen(void)
{
	PMD_InitTypeDef	pmdini;

	pmdini.shunt = cSHUNT_TYPE_CH1;
	pmdini.poll = cPOLL_CH1;
	pmdini.polh = cPOLH_CH1;
	pmdini.pwmfreq = PWMPeriod(cPWMPRD_CH1);
	pmdini.deadtime = DeadTime(cDEADTIME_CH1);

	IP_PMD_init(TSB_PMD1, &pmdini);

	/* Setting interrupt. */
	NVIC_SetPriority(INTPMD1_IRQn, INT_VC_LEVEL+1);
	NVIC_ClearPendingIRQ(INTPMD1_IRQn);
	NVIC_DisableIRQ(INTPMD1_IRQn);
}


/**
 *********************************************************************************************
 * @brief		Initialize VE.
 *
 * @return		none
 *********************************************************************************************
 */
void init_VEen(void)
{
	VE_InitTypeDef	veini;

	veini.shunt = cSHUNT_TYPE_CH1;
	veini.pwmfreq = PWMPeriod(cPWMPRD_CH1);
	veini.reptime = cREPTIME_CH1;
	veini.trgmode = cTRGMODE1_INI;
	veini.tpwm = (uint16_t)(cPWMPRD_CH1 * 1e-6 * cHZ_MAX_CH1 * (1 << 16U));
	veini.idkp = (uint16_t)(q15_t)(FIX_15 * cID_KP_CH1 * cA_MAX_CH1 / cV_MAX_CH1);
	veini.idki = (uint16_t)(q15_t)(FIX_15 * cID_KI_CH1 * cCTRL_PRD_CH1 * cA_MAX_CH1 / cV_MAX_CH1);
	veini.iqkp = (uint16_t)(q15_t)(FIX_15 * cIQ_KP_CH1 * cA_MAX_CH1 / cV_MAX_CH1);
	veini.iqki = (uint16_t)(q15_t)(FIX_15 * cIQ_KI_CH1 * cCTRL_PRD_CH1 * cA_MAX_CH1 / cV_MAX_CH1);
	veini.zerooffset = ADC_V2HEX(cSHUNT_ZERO_OFFSET_CH1);

	NVIC_SetPriority(INTVCNB_IRQn, INT_VC_LEVEL);		/* Interruption level set */
	NVIC_ClearPendingIRQ(INTVCNB_IRQn);
	NVIC_EnableIRQ(INTVCNB_IRQn);						/* INTVCNx enable */

	IP_VE_init(TSB_VE, &veini);

}

/**
 *********************************************************************************************
 * @brief		Initialize WDT
 *
 * @return		none
 *********************************************************************************************
 */
void init_WDT(void)
{

	WDT_InitTypeDef WDTini;

	WDTini.DetectTime = WDT_DETECT_TIME_EXP_25;
	WDTini.OverflowOutput = WDT_WDOUT;

	WDT_Init(&WDTini);
	WDT_WriteClearCode();
	WDT_Enable();

}

/*********************************** END OF FILE ******************************/
