/**
 ****************************************************************************
 * @file	interrupt.c
 * @brief	Interrupt Functions for VE+
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>

#include "ipdefine.h"

#include "mcuip_drv.h"

#include "D_Driver.h"
#include "usercon.h"

#define DEFINE_APP
#include "interrupt.h"
#undef	DEFINE_APP


/**
 *********************************************************************************************
 * @brief		INT_interval_4kHz
 *
 * @return		none
 *********************************************************************************************
 */
void INT_interval_4kHz(void)
{
	M_Main_Counter++;									/* Count for main routine Period */

}

/**
 *********************************************************************************************
 * @brief		INT_VCN1
 *
 * @return		none
 *********************************************************************************************
 */
void INT_VCN1(void)
{

	vector_t* const _motor = &Motor_ch1;
	TSB_VE_TypeDef* const VEx = TSB_VE;

#if (__FIXED_VDC_CH1 == 1)								/* 1: Vdc from constant data */
	/***** Set Vdc data to VE *****/
	VE_SetVDCreg(VEx, (q15_t)(FIX_15 * cVDC_CH1 / cV_MAX_CH1));
#endif

	/***** Get data from VE register *****/
	VE_GetdataFromVEreg(VEx, _motor);

	/***** Check over current *****/
	VE_GetPhaseCurrent(VEx, &_motor->drv.Ia, &_motor->drv.Ib, &_motor->drv.Ic);
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
        _motor->drv.state.flg.emg_S = SET;
        _motor->stage.itr = ciEmergency;
	}

	/***** Check over voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH1 / cV_MAX_CH1)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH1 / cV_MAX_CH1) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER)
		/***** Encoder *****/
		H_Encoder(_motor, TSB_EN0);
#endif /* __USE_ENCODER */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH1)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}

	}

	/***** Check Current detection error  *****/
#if (cSHUNT_TYPE_CH1 == 1)
	if (  (VE_GetOutputMode(VEx) == OCRMD_OUT_ON)
	   && (D_Check_DetectCurrentError_1shunt(	 VE_GetPWM_DutyU(VEx),
												VE_GetPWM_DutyV(VEx),
												VE_GetPWM_DutyW(VEx),
												_motor->drv.chkpls,
												VE_GetShiftPWMState(VEx),
												VE_GetPWM_Modulation(VEx),
												VE_GetSector(VEx),
												VE_GetPWM_Period(VEx)) != 0)
	)
	{
		_motor->drv.idetect_error = 1;
	}
	else
	{
		_motor->drv.idetect_error = 0;
	}
#elif (cSHUNT_TYPE_CH1 == 3)
	if (  (VE_GetOutputMode(VEx) == OCRMD_OUT_ON)
	   && (D_Check_DetectCurrentError_3shunt( VE_GetPWM_DutyMed(VEx), _motor->drv.chkpls) != 0)
	)
	{
		_motor->drv.idetect_error = 1;
	}
	else
	{
		_motor->drv.idetect_error = 0;
	}
#endif /* cSHUNT_TYPE_CH1 */

	/**** Set data to VE register *****/
	/* Set Shift pwm change speed. for 1shunt */
	VE_SetSpwmMode(VEx, _motor->drv.command.spwm);

	/* Set modulation type */
	VE_SetModulType(VEx, _motor->drv.command.modul);

	switch (_motor->stage.itr)
	{
			/*--- Stop Stage -----------------------------*/
		case ciStop:
			VE_SetdataToVEreg_Stop(VEx, _motor);
			break;

			/*--- Bootstrap Stage ------------------------*/
		case ciBootstrap:
			VE_SetdataToVEreg_Bootstrap(VEx, _motor);
			break;

			/*--- Position Stage (Current type) ----------*/
		case ciInitposition_i:
			VE_SetdataToVEreg_Initposition_i(VEx, _motor);
			break;

			/*--- Position Stage (Voltage type) ----------*/
		case ciInitposition_v:
			VE_SetdataToVEreg_Initposition_v(VEx, _motor);
			break;

			/*--- Force Stage (Current type) -------------*/
		case ciForce_i:
			VE_SetdataToVEreg_Force_i(VEx, _motor);
			break;

			/*--- Force Stage (Voltage type) -------------*/
		case ciForce_v:
			VE_SetdataToVEreg_Force_v(VEx, _motor);
			break;

			/*--- ChangeUp Stage -------------------------*/
		case ciChange_up:
			VE_SetdataToVEreg_Change_up(VEx, _motor);
			break;

			/*--- Steady_A Stage ---------------------------*/
		case ciSteady_A:
			VE_SetdataToVEreg_Steady_A(VEx, _motor);
			break;

			/*--- Emergency Stage ------------------------*/
		case ciEmergency:
			VE_SetdataToVEreg_Emergency(VEx, _motor);
			break;
			/*--- Fail-safe ------------------------*/
		default:
			VE_SetdataToVEreg_Stop(VEx, _motor);
			break;
	}

#if defined(__USE_DAC)
	UiOutDataStart();									/* Dac output for debug */
#endif /* __USE_DAC */

	VE_Start(VEx);

}


/**
 *********************************************************************************************
 * @brief		INT_PWM1
 *
 * @return		none
 *********************************************************************************************
 */
void	INT_PWM1(void)
{
}

/**
 *********************************************************************************************
 * @brief		INT_ADB
 *
 * @return		none
 *********************************************************************************************
 */
void	INT_ADB(void)
{
}

/*********************************** END OF FILE ******************************/
