/**
 ****************************************************************************
 * @file	usercon.c
 * @brief	User Control
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>

#include "ipdefine.h"
#include "D_Para.h"
#include "tmpm375_gpio.h"
#include "tmpm375_adc.h"
#include "tmpm375_uart.h"
#include "dac_drv.h"

#define DEFINE_APP
#include "usercon.h"
#undef	DEFINE_APP


/*===================================================================*
	  Macro Definition
 *===================================================================*/
/* On this system, VDC and VR1 are assigned to the same ADC_AIN9.
   VR1 is enabled only when Vdc is obtained from the constant data. */
#define __USE_VR1           (__FIXED_VDC_CH1)               /* 0 : VR1 not available, 1 : VR1 available */

/*===================================================================*
	  Local Constant Data Definition
 *===================================================================*/
/* Key setting */
#define P_TSW1				TSB_PB_DATA_PB5 				/* TSW1 */
#define P_SW1				TSB_PB_DATA_PB6 				/* SW1 */
#define cKEY_CHATA_CNT		(20)							/* [cnt] chattering counter for KEY SW */

/* Soft ADC Setting */
#define	cADUNIT_USR			TSB_ADB							/* User ad data ADCUnit */
#define cADCH_VR			ADC_AIN9						/* ADC Channel for VR */
#define cADREG_VR			ADC_REG7						/* Result register No for VR */
#define cADAVECNT			(10)							/* ADC average count */

/* LED setting */
#define P_LED5				TSB_PF_DATA_PF0                 /* LED5 */

/* UART setting */
#define SIOCH_PCCOM			TSB_SC1                         /* UART Channel for PC COM */

/*===================================================================*
	  Typedef Definition
 *===================================================================*/
typedef struct
{
	uint8_t		select;										/* Dac output select */
	uint8_t		motch;										/* Dac motor ch select */
	uint8_t		datsft0;									/* Dac data shift for data0 */
	uint8_t		datsft1;									/* Dac data shift for data1 */
	uint8_t		datsft2;									/* Dac data shift for data2 */
	uint8_t		datsft3;									/* Dac data shift for data3 */
} dac_t;


extern	int32_t	target_spd1;


/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
static sw_t			keyportbuf_old; 						/* Previous Key port data */
static uint8_t		key_chatacnt;							/* Chattering counter for Key */
static addat_t		ad_vr1;									/* AD value of volume register */
static int32_t		vr1_speed;								/* VR1 speed data */
static uint8_t		motor_on_ok;
static uint8_t		sw_chg_stop;

/* No, ch, sft0, sft1, sft2, sft3 */
 dac_t 		dac = {   0,  1,	0,	  0,	0,	  0 };



/*===================================================================*
	  Local Proto Type Definition
 *===================================================================*/
static void init_Uikey(void);
static void Uikeyscan(void);
static void init_soft_adc(void);
static void soft_adc_getdata(void);
#if (__USE_VR1 == 1)	
static void soft_adc_avecal(addat_t* const _ad_dat, uint16_t _rawdat);
#endif
static void vr1speed(void);
static void init_led(void);
static void led_control(void);


/**
 *********************************************************************************************
 * @brief		init_user_control
 *
 * @return		none
 *********************************************************************************************
 */
void	init_user_control(void)
{

	init_Uikey();										/* Initial setting of key input */
	init_soft_adc();									/* Initial setting of soft adc */
    init_led();                                         /* Initial setting of led output */
}

/**
 *********************************************************************************************
 * @brief		user_control
 *
 * @return		none
 *********************************************************************************************
 */
void	user_control(void)
{
	/***** Get AD Data *****/
	soft_adc_getdata();

	/***** key input *****/
	Uikeyscan();

	/***** Set Target Speed *****/
	vr1speed();

	if (motor_on_ok != SET)
	{
		/* Process to make the motor startup possible 
		   after detecting the target speed becoming 0 once after starting the microcomputer */
		if ((vr1_speed == 0) && (ad_vr1.detflg == SET))
	    {
			motor_on_ok = SET;
		}
	}
	if (motor_on_ok == SET)
    {
#if (__USE_VR1 == 1)	
		target_spd1 = vr1_speed;
#endif
    }

	/***** LED Control *****/
    led_control();
}

/**
 *********************************************************************************************
 * @brief		Initialize key.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_Uikey(void)
{
	/* TSW1 */
	GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_5, DISABLE);
	GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_5, ENABLE);
	GPIO_DisableFuncReg(GPIO_PB, GPIO_FUNC_REG_1, GPIO_BIT_5);

#if (__LED_SW == 1)
	/* SW1 */
	GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_6, DISABLE);
	GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_6, ENABLE);
	GPIO_DisableFuncReg(GPIO_PB, GPIO_FUNC_REG_1, GPIO_BIT_6);
#endif

}

/**
 *********************************************************************************************
 * @brief		keyscan
 *
 * @return		none
 *********************************************************************************************
 */
static void Uikeyscan(void)
{
	sw_t	portbuf;

	*((uint8_t*)&portbuf) = 0;

	portbuf.tsw1 = P_TSW1;

#if (__LED_SW == 1)
	portbuf.sw1 = ~P_SW1;
#endif

	if (*((uint8_t*)&portbuf) == *((uint8_t*)&keyportbuf_old))
	{
		if (key_chatacnt != 0)
		{
			if (--key_chatacnt == 0)
			{
				*((uint8_t*)&keydata) = *((uint8_t*)&portbuf);
				sw_chg_stop = 1;
			}
		}
	}
	else
	{
		*((uint8_t*)&keyportbuf_old) = *((uint8_t*)&portbuf);
		key_chatacnt = cKEY_CHATA_CNT;
	}
}

/**
 *********************************************************************************************
 * @brief		Initialize softconvert adc.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_soft_adc(void)
{
#if (__USE_VR1 == 1)
	ADC_SetSWTrg(cADUNIT_USR, cADREG_VR, TRG_ENABLE(cADCH_VR));
#endif
}


/**
 *********************************************************************************************
 * @brief		soft_adc_getdata
 *
 * @return		none
 *********************************************************************************************
 */
static void soft_adc_getdata(void)
{
#if (__USE_VR1 == 1)
	if (ADC_GetConvertState(cADUNIT_USR, ADC_TRG_SW) == DONE)		/* finish adc? */
	{
  		ADC_Result tmp = ADC_GetConvertResult(cADUNIT_USR, cADREG_VR);
		soft_adc_avecal(&ad_vr1, (uint16_t)tmp.Bit.ADResult);
	}
	ADC_Start(cADUNIT_USR, ADC_TRG_SW); 							/* adc start */
#endif
}


/**
 *********************************************************************************************
 * @brief		soft_adc_avecal
 *
 * @param		addat_t* const	_ad_dat
 * @param		uint16_t _rawdat
 *
 * @return		none
 *********************************************************************************************
 */
#if (__USE_VR1 == 1)
static void soft_adc_avecal(addat_t* const _ad_dat, uint16_t _rawdat)
{

	if (_ad_dat->avecnt == 0)
	{
		/* first? */
		_ad_dat->maxdat = _rawdat;
		_ad_dat->mindat = _rawdat;
	}
	else
	{
		if (_rawdat > _ad_dat->maxdat)
		{
			_ad_dat->maxdat = _rawdat;
		}
		else if (_rawdat < _ad_dat->mindat)
		{
			_ad_dat->mindat = _rawdat;
		}
	}

	_ad_dat->sumdat += _rawdat;

	_ad_dat->avecnt++;
	if (_ad_dat->avecnt >= cADAVECNT)
	{
		_ad_dat->avecnt = 0;
		_ad_dat->sumdat -= _ad_dat->maxdat;
		_ad_dat->sumdat -= _ad_dat->mindat;
		_ad_dat->avedat = (_ad_dat->sumdat / (cADAVECNT - 2));
		_ad_dat->sumdat = 0;
		_ad_dat->detflg = SET;
	}

}
#endif

/****************************************************************************************
 * Module	: vr1speed
 *---------------------------------------------------------------------------------------
 * Function : none
 * Input	: none
 * Output	: none
 * Note 	:
 ****************************************************************************************/
#define		cAD_MIN		0x10
#define		cAD_MAX		0xF0
static void vr1speed(void)
{
	int32_t speed_temp = 0;
	uint32_t ad_temp = (uint32_t)(ad_vr1.avedat >> 4);

	if (ad_temp < cAD_MIN)
	{
		speed_temp = 0;
	}
	else if (ad_temp < cAD_MAX)
	{
		speed_temp = (cSPEED_ACT_MAX - cSPEED_ACT_MIN) * (ad_temp - cAD_MIN) / (cAD_MAX - cAD_MIN) + cSPEED_ACT_MIN;
	}
	else
	{
		speed_temp = cSPEED_ACT_MAX;
	}

	if(sw_chg_stop == 1){
		if(speed_temp != 0){
			speed_temp = 0;
		}
		else{
			sw_chg_stop = 0;
		}
	}

    if (keydata.tsw1 == 1)
    {
        vr1_speed = speed_temp;  /* cw */
    }
    else
    {
        vr1_speed = -speed_temp; /* ccw */
    }
}

/**
 *********************************************************************************************
 * @brief		Initialize LED output
 *
 * @return		none
 *********************************************************************************************
 */
static void init_led(void)
{
#if (__LED_SW == 1)
   	/* LED5 */
    GPIO_WriteDataBit(GPIO_PF, GPIO_BIT_0, GPIO_BIT_VALUE_1);
	GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_0, ENABLE);
#endif
}

/**
 *********************************************************************************************
 * @brief		LED output control
 *
 * @return		none
 *********************************************************************************************
 */
static void led_control(void)
{
#if (__LED_SW == 1)
    /* LED5 control */
    if (Motor_ch1.stage.main == cEmergency)
    {
		P_LED5 = 0;		/* LED ON */
    }else{
		P_LED5 = 1;		/* LED OFF */
    }
#endif
}

/**
 *********************************************************************************************
 * @brief		Initialize UartDrv
 *
 * @return		none
 *********************************************************************************************
 */
void init_UartDrv(void)
{
#if (__PC_CH1 == 1)

	UART_DefaultConfig(SIOCH_PCCOM);

	GPIO_SetOutputEnableReg(GPIO_PB, GPIO_BIT_6, DISABLE);
	GPIO_SetInputEnableReg(GPIO_PB, GPIO_BIT_6, ENABLE);
	GPIO_EnableFuncReg(GPIO_PB, GPIO_FUNC_REG_5, GPIO_BIT_6);

	GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_0, ENABLE);
	GPIO_SetInputEnableReg(GPIO_PF, GPIO_BIT_0, DISABLE);
	GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_5, GPIO_BIT_0);

	NVIC_SetPriority(INTRX1_IRQn, 7); 	/* Interruption level set */
	NVIC_ClearPendingIRQ(INTRX1_IRQn);
	NVIC_EnableIRQ(INTRX1_IRQn);		/* INTRX enable */
#endif
}

/****************************************************************************************
 * Module   : UartDrv_putc
 *---------------------------------------------------------------------------------------
 * Function : PC(UART) Send
 * Input    : Txbuff: the data will be sent
 * Output   : none
 * Note     : 
 ****************************************************************************************/
void UartDrv_putc(char c)
{
#if (__PC_CH1 == 1)

	while (UART_GetBufState(SIOCH_PCCOM, UART_TX) != DONE)
	{
		/* wait empty */
	}
	UART_SetTxData(SIOCH_PCCOM, (uint32_t)c);

#endif
}


/****************************************************************************************
 * Module   : UartDrv_puts
 *---------------------------------------------------------------------------------------
 * Function : String send
 * Input    : String pointer
 * Output   : none
 * Note     : 
 ****************************************************************************************/
void UartDrv_puts(const char *str)
{
#if (__PC_CH1 == 1)

	while ( *str != '\0' )								/* !String End */
	{
		UartDrv_putc( *str );							/* TX Data Write */
		str++;											/* Next Data Point */
	}

#endif
}

/****************************************************************************************
 * Module   : UartDrv_getc
 *---------------------------------------------------------------------------------------
 * Function : PC(UART) Receive
 * Input    : none
 * Output   : Rxbuff : the data received.
 * Note     : 
 ****************************************************************************************/
char UartDrv_getc(void)
{
#if (__PC_CH1 == 1)

	if (UART_GetBufState(SIOCH_PCCOM, UART_RX) == DONE)
	{
		return ((char)UART_GetRxData(SIOCH_PCCOM));
	}
	else
	{
		return ('\0');
	}
#else
		return ('\0');
#endif
}

/****************************************************************************************
 * Module   : UartDrvRcv
 *---------------------------------------------------------------------------------------
 * Function : PC(UART) Receive Interrupt
 * Input    : none
 * Output   : none
 * Note     : 
 ****************************************************************************************/
void UartDrvRcv(void)
{
#if (__PC_CH1 == 1)
    static volatile char RxData = '\0'; /* for test */
    RxData = UartDrv_getc();
#endif    
}
/**
 *********************************************************************************************
 * @brief		UiOutDataStart
 *
 *
 * @return		none
 *********************************************************************************************
 */
void UiOutDataStart(void)
{
	dac_t* const pdac = &dac;
	{
		vector_t* pmotor = &Motor_ch1;
		TSB_VE_TypeDef* pVE = TSB_VE;
		uint16_t* const pdacout = DacData;

		switch (pdac->select)
		{
			case 0:
				*(pdacout + 0) = (pVE->TMPREG0 >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pVE->TMPREG1 >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 2) = (pVE->TMPREG2 >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 3) = pmotor->drv.theta.half[1];
				break;
			case 1:
				*(pdacout + 0) = (pmotor->drv.Id_ref << pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Id	 << pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Iq_ref << pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq	 << pdac->datsft0) ^ 0x8000;
				break;
			case 2:
				*(pdacout + 0) = (pmotor->drv.omega_com.half[1] << pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.omega.half[1] 	<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.omega_dev 		<< pdac->datsft1) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq_ref			<< pdac->datsft2) ^ 0x8000;
				break;
			case 3:
				*(pdacout + 0) = (pVE->TMPREG0              >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Iq_ref        << pdac->datsft1)        ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Id_ref        << pdac->datsft1)        ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.omega.half[1] << pdac->datsft2)        ^ 0x8000;
				break;
			default:
				*(pdacout + 0) = *(pdacout + 0) + 0x0100;
				*(pdacout + 1) = *(pdacout + 1) + 0x0200;
				*(pdacout + 2) = *(pdacout + 2) + 0x0300;
				*(pdacout + 3) = *(pdacout + 3) + 0x0400;
				break;
		}
		DacWriteStart(DAC_SIO_CH);
	}
}

/******************************** END OF FILE *********************************/
